@@ -4,7 +4,7 @@ use strict;
 use Carp;
 use vars qw($VERSION $PACKAGE @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '0.21';
+$VERSION = '0.22';
 $PACKAGE = 'ShiftJIS::Regexp'; #__PACKAGE__
 
 require Exporter;
@@ -179,6 +179,20 @@ my %Re = (
 		. '\x88[\x40-\x7E\x80-\x9E]|\x98[\x73-\x7E\x80-\x9E]|'
 		. '\xEA[\xA5-\xFC]'.$Close,
 
+  '\p{halfwidth}' => '[\x21\x23-\x26\x28-\x2C\x2E-\x7E]',
+  '\P{halfwidth}' => $Open.'[\x00-\x20\x22\x27\x2D\x7F\xA1-\xDF]|'.$DBC.$Close,
+
+  '\p{fullwidth}' => $Open.'\x81[\x43\x44\x46-\x49\x4d\x4f\x50\x51\x5e'
+		. '\x62\x69\x6a\x6d-\x70\x7b\x81\x83\x84\x8f\x90\x93-\x97]|'
+		. '\x82[\x4f-\x58\x60-\x79\x81-\x9a]'.$Close,
+
+  '\P{fullwidth}' => $Open.$SBC
+		. '|\x81[\x40-\x42\x45\x4a-\x4c\x4e\x52-\x5d\x5f-\x61'
+		. '\x63-\x68\x6b\x6c\x71-\x7a\x7c-\x7e\x80\x82\x85-\x8e'
+		. '\x91\x92\x98-\xfc]'
+		. '|\x82[\x40-\x4e\x59-\x5f\x7a-\x7e\x80\x9b-\xfc]'
+		. '|[\x83-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'. $Close,
+
   '\p{hankaku}' => $Open.'[\xA1-\xDF]'.$Close,
   '\P{hankaku}' => $Open.'[\x00-\x7F]|' .$DBC.$Close,
   '\p{zenkaku}' => "$Open$DBC$Close",
@@ -196,24 +210,23 @@ my %Re = (
 		. '\xEA[\x40-\x7E\x80-\xA4]'
 		.$Close,
 
-  '\P{x0208}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '\x81[\xAD-\xB7\xC0-\xC7\xCF-\xD9\xE9-\xEF\xF8-\xFB]|'
-		. '\x82[\x40-\x4E\x59-\x5F\x7A-\x7E\x80\x9B-\x9E\xF2-\xFC]|'
-		. '\x83[\x97-\x9E\xB7-\xBE\xD7-\xFC]|'
-		. '\x84[\x61-\x6F\x92-\x9E\xBF-\xFC]|'
-		. '[\x85-\x87\xEB-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x88[\x40-\x7E\x80-\x9E]|\x98[\x73-\x7E\x80-\x9E]|'
-		. '\xEA[\xA5-\xFC]'.$Close,
+  '\P{x0208}' => $Open.$SBC
+		. '|\x81[\xAD-\xB7\xC0-\xC7\xCF-\xD9\xE9-\xEF\xF8-\xFB]'
+		. '|\x82[\x40-\x4E\x59-\x5F\x7A-\x7E\x80\x9B-\x9E\xF2-\xFC]'
+		. '|\x83[\x97-\x9E\xB7-\xBE\xD7-\xFC]'
+		. '|\x84[\x61-\x6F\x92-\x9E\xBF-\xFC]'
+		. '|[\x85-\x87\xEB-\xFC][\x40-\x7E\x80-\xFC]'
+		. '|\x88[\x40-\x7E\x80-\x9E]|\x98[\x73-\x7E\x80-\x9E]'
+		. '|\xEA[\xA5-\xFC]'.$Close,
 
-  '\p{jis}'   => $Open.'[\x00-\x7F\xA1-\xDF]|\x81[\x40-\x7E'
+  '\p{jis}'   => $Open.$SBC.'|\x81[\x40-\x7E'
 		. '\x80-\xAC\xB8-\xBF\xC8-\xCE\xDA-\xE8\xF0-\xF7\xFC]|'
 		. '\x82[\x4F-\x58\x60-\x79\x81-\x9A\x9F-\xF1]|'
 		. '\x83[\x40-\x7E\x80-\x96\x9F-\xB6\xBF-\xD6]|'
 		. '\x84[\x40-\x60\x70-\x7E\x80-\x91\x9F-\xBE]|'
 		. '\x88[\x9F-\xFC]|\x98[\x40-\x72\x9F-\xFC]|'
 		. '[\x89-\x97\x99-\x9F\xE0-\xE9][\x40-\x7E\x80-\xFC]|'
-		. '\xEA[\x40-\x7E\x80-\xA4]'
-		. $Close,
+		. '\xEA[\x40-\x7E\x80-\xA4]'. $Close,
 
   '\P{jis}'   => $Open
 		. '\x81[\xAD-\xB7\xC0-\xC7\xCF-\xD9\xE9-\xEF\xF8-\xFB]|'
@@ -229,22 +242,16 @@ my %Re = (
   '\P{latin}' => $Open.'[\x00-\x40\x5B-\x60\x7B-\x7F\xA1-\xDF]|'.$DBC.$Close,
 
   '\p{fulllatin}' => $Open.'\x82[\x60-\x79\x81-\x9A]'.$Close,
-  '\P{fulllatin}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x81\x83-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x82[\x40-\x5F\x7A-\x7E\x80\x9B-\xFC]'
-		. $Close,
+  '\P{fulllatin}' => $Open.$SBC.'|\x82[\x40-\x5F\x7A-\x7E\x80\x9B-\xFC]|'
+		. '[\x81\x83-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'. $Close,
 
   '\p{greek}' => $Open.'\x83[\x9f-\xb6\xbf-\xd6]'.$Close,
-  '\P{greek}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x81\x82\x84-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x83[\x40-\x7E\x80-\x9e\xb7-\xbe\xd7-\xFC]'
-		. $Close,
+  '\P{greek}' => $Open.$SBC.'|\x83[\x40-\x7E\x80-\x9e\xb7-\xbe\xd7-\xFC]|'
+		. '[\x81\x82\x84-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'. $Close,
 
   '\p{cyrillic}' => $Open.'\x84[\x40-\x60\x70-\x7E\x80-\x91]'.$Close,
-  '\P{cyrillic}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x81-\x83\x85-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x84[\x61-\x6f\x92-\xFC]'
-		. $Close,
+  '\P{cyrillic}' => $Open.$SBC.'|\x84[\x61-\x6f\x92-\xFC]|'
+		. '[\x81-\x83\x85-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'. $Close,
 
   '\p{european}' => $Open.'[\x41-\x5A\x61-\x7A]|\x82[\x60-\x79\x81-\x9A]|'
 		. '\x83[\x9f-\xb6\xbf-\xd6]|\x84[\x40-\x60\x70-\x7E\x80-\x91]'
@@ -260,25 +267,21 @@ my %Re = (
   '\P{halfkana}' => $Open.'[\x00-\x7F\xA1-\xA5]|' .$DBC.$Close,
 
   '\p{hiragana}' => $Open.'\x82[\x9F-\xF1]|\x81[\x4A\x4B\x54\x55]'.$Close,
-  '\P{hiragana}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x83-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x82[\x40-\x7E\x80-\x9E\xF2-\xFC]|'
-		. '\x81[\x40-\x49\x4C-\x53\x56-\x7E\x80-\xFC]'
-		. $Close,
+  '\P{hiragana}' => $Open.$SBC.'|[\x83-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'
+		. '|\x82[\x40-\x7E\x80-\x9E\xF2-\xFC]'
+		. '|\x81[\x40-\x49\x4C-\x53\x56-\x7E\x80-\xFC]'. $Close,
 
   '\p{katakana}' => $Open.'\x83[\x40-\x7E\x80-\x96]|\x81[\x52\x53\x5B]'.$Close,
-  '\P{katakana}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x82\x84-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x83[\x97-\xFC]|'
-		. '\x81[\x40-\x51\x54-\x5A\x5C-\x7E\x80-\xFC]'
+  '\P{katakana}' => $Open.$SBC
+		. '|[\x82\x84-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'
+		. '|\x83[\x97-\xFC]|\x81[\x40-\x51\x54-\x5A\x5C-\x7E\x80-\xFC]'
 		. $Close,
 
   '\p{fullkana}' => $Open.'\x82[\x9F-\xF1]|\x83[\x40-\x7E\x80-\x96]|'
 		    . '\x81[\x4A\x4B\x5B\x52-\x55]'.$Close,
-  '\P{fullkana}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x84-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x82[\x40-\x7E\x80-\x9E\xF2-\xFC]|\x83[\x97-\xFC]|'
-		. '\x81[\x40-\x49\x4C-\x51\x56-\x5A\x5C-\x7E\x80-\xFC]'
+  '\P{fullkana}' => $Open.$SBC.'|[\x84-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'
+		. '|\x82[\x40-\x7E\x80-\x9E\xF2-\xFC]|\x83[\x97-\xFC]'
+		. '|\x81[\x40-\x49\x4C-\x51\x56-\x5A\x5C-\x7E\x80-\xFC]'
 		. $Close,
 
   '\p{kana}' => $Open.'[\xA6-\xDF]|\x82[\x9F-\xF1]|\x83[\x40-\x7E\x80-\x96]|'
@@ -290,51 +293,47 @@ my %Re = (
 		. $Close,
 
   '\p{kanji0}'  => $Open.'\x81[\x56-\x5A]'.$Close,
-  '\P{kanji0}'  => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '\x81[\x40-\x55\x5b-\x7E\x80-\xFC]|'
-		. '[\x82-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'
-		. $Close,
+  '\P{kanji0}'  => $Open.$SBC.'|\x81[\x40-\x55\x5b-\x7E\x80-\xFC]'
+		. '|[\x82-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'. $Close,
 
   '\p{kanji1}'  => $Open.'\x88[\x9F-\xFC]|\x98[\x40-\x72]|'
 		. '[\x89-\x97][\x40-\x7E\x80-\xFC]'.$Close,
-  '\P{kanji1}'  => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '\x88[\x40-\x7E\x80-\x9E]|\x98[\x73-\x7E\x80-\xFC]|'
+  '\P{kanji1}'  => $Open.$SBC.'|\x88[\x40-\x7E\x80-\x9E]|'
+		. '\x98[\x73-\x7E\x80-\xFC]|'
 		. '[\x81-\x87\x99-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'
 		. $Close,
 
   '\p{kanji2}'  => $Open.'\x98[\x9F-\xFC]|[\x99-\x9F\xE0-\xE9]'
 		. '[\x40-\x7E\x80-\xFC]|\xEA[\x40-\x7E\x80-\xA4]'
 		. $Close,
-  '\P{kanji2}'  => $Open.'[\x00-\x7F\xA1-\xDF]|\x98[\x40-\x7E\x80-\x9E]|'
+  '\P{kanji2}'  => $Open.$SBC.'|\x98[\x40-\x7E\x80-\x9E]|'
 		. '[\x81-\x97\xEB-\xFC][\x40-\x7E\x80-\xFC]|\xEA[\xA5-\xFC]'
 		. $Close,
 
-  '\p{kanji}'   => $Open.'\x81[\x56-\x5A]|\x88[\x9F-\xFC]|'
-		. '[\x89-\x97\x99-\x9F\xE0-\xE9][\x40-\x7E\x80-\xFC]|'
-		. '\x98[\x40-\x72\x9F-\xFC]|\xEA[\x40-\x7E\x80-\xA4]'
-		. $Close,
-  '\P{kanji}'   => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '\x81[\x40-\x55\x5b-\x7E\x80-\xFC]|'
-		. '\x88[\x40-\x7E\x80-\x9E]|\x98[\x73-\x7E\x80-\x9E]|'
-		. '[\x82-\x87\xEB-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\xEA[\xA5-\xFC]'
+  '\p{kanji}'   => $Open.'\x81[\x56-\x5A]|\x88[\x9F-\xFC]'
+		. '|[\x89-\x97\x99-\x9F\xE0-\xE9][\x40-\x7E\x80-\xFC]'
+		. '|\x98[\x40-\x72\x9F-\xFC]|\xEA[\x40-\x7E\x80-\xA4]'
 		. $Close,
+  '\P{kanji}'   => $Open.$SBC
+		. '|\x81[\x40-\x55\x5b-\x7E\x80-\xFC]'
+		. '|\x88[\x40-\x7E\x80-\x9E]|\x98[\x73-\x7E\x80-\x9E]'
+		. '|[\x82-\x87\xEB-\xFC][\x40-\x7E\x80-\xFC]'
+		. '|\xEA[\xA5-\xFC]'. $Close,
 
   '\p{boxdrawing}' => $Open.'\x84[\x9F-\xBE]'.$Close,
-  '\P{boxdrawing}' => $Open.'[\x00-\x7F\xA1-\xDF]|'
-		. '[\x81-\x83\x85-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]|'
-		. '\x84[\x40-\x7E\x80-\x9E\xBF-\xFC]'
-		. $Close,
+  '\P{boxdrawing}' => $Open.$SBC
+		. '|[\x81-\x83\x85-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]'
+		. '|\x84[\x40-\x7E\x80-\x9E\xBF-\xFC]'. $Close,
 
-  '\p{nec}' => $Open. '\x87[\x40-\x5d\x5f-\x75\x7e\x80-\x9c]|'
-		. '\xed[\x40-\x7e\x80-\xfc]|\xee[\x40-\x7e\x80-\xec\xef-\xfc]'
+  '\p{nec}' => $Open. '\x87[\x40-\x5d\x5f-\x75\x7e\x80-\x9c]'
+		. '|\xed[\x40-\x7e\x80-\xfc]|\xee[\x40-\x7e\x80-\xec\xef-\xfc]'
 		. $Close,
 
   '\p{ibm}' => $Open.'[\xfa-\xfb][\x40-\x7e\x80-\xfc]|\xfc[\x40-\x4b]'.$Close,
 
-  '\p{vendor}' => $Open. '\x87[\x40-\x5d\x5f-\x75\x7e\x80-\x9c]|'
-		. '[\xed\xfa-\xfb][\x40-\x7e\x80-\xfc]|'
-		. '\xee[\x40-\x7e\x80-\xec\xef-\xfc]|\xfc[\x40-\x4b]'
+  '\p{vendor}' => $Open. '\x87[\x40-\x5d\x5f-\x75\x7e\x80-\x9c]'
+		. '|[\xed\xfa-\xfb][\x40-\x7e\x80-\xfc]'
+		. '|\xee[\x40-\x7e\x80-\xec\xef-\xfc]|\xfc[\x40-\x4b]'
 		. $Close,
 
   '\p{mswin}' => $Open.'[\x00-\x7f\xa1-\xdf]|'
@@ -372,6 +371,7 @@ my %AbbrevProp = qw(
   R  roman
   Y  hankaku
   Z  zenkaku
+  F  fullwidth
   J  jis
   N  nec
   I  ibm
@@ -528,7 +528,8 @@ sub _parse_class {
           }
         } else {
           push(@re, ']');
-          $state = 1;
+    
+      $state = 1;
           next;
         }
         $state = 4;
@@ -1370,7 +1371,6 @@ and returns the array given by split of the specified string into characters.
    \S                        [\S]              [^\t\n\r\f ]
 
    \p{Xdigit}     \pX        [[:xdigit:]]      [0-9A-Fa-f]
-
    \p{Digit}      \pD        [[:digit:]]       [0-9O-X]
    \p{Upper}      \pU        [[:upper:]]       [A-Z`-y]
    \p{Lower}      \pL        [[:lower:]]       [a-z-]
@@ -1380,20 +1380,21 @@ and returns the array given by split of the specified string into characters.
    \p{Word}       \pW        [[:word:]]        [_\p{Digit}\p{European}\p{Kana}\p{Kanji}]
    \p{Punct}      \pP        [[:punct:]]       [!-/:-@[-`{-~-A-IL-Q\---΁---]
    \p{Graph}      \pG        [[:graph:]]       [\p{Word}\p{Punct}]
-   \p{Blank}      \pB        [[:blank:]]       [\x20\x{8140}\t]
    \p{Print}      \pT        [[:print:]]       [\x20\x{8140}\p{Graph}]
    \p{Space}      \pS        [[:space:]]       [\x20\x{8140}\x09-\x0D]
+   \p{Blank}      \pB        [[:blank:]]       [\x20\x{8140}\t]
    \p{Cntrl}      \pC        [[:cntrl:]]       [\x00-\x1F\x7F]
 
    \p{Roman}      \pR        [[:roman:]]       [\x00-\x7F]
    \p{ASCII}                 [[:ascii:]]       [\p{Roman}]
    \p{Hankaku}    \pY        [[:hankaku:]]     [\xA1-\xDF]
    \p{Zenkaku}    \pZ        [[:zenkaku:]]     [\x{8140}-\x{FCFC}]
+   \p{Halfwidth}             [[:halfwidth:]]   [!#$%&()*+,./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~]
+   \p{Fullwidth}  \pF        [[:fullwidth:]]   [Iij{CD^O-XFGH`-ymnOQM-obpP]
 
    \p{X0201}                 [[:x0201:]]       [\x00-\x7F\xA1-\xDF]
    \p{X0208}                 [[:x0208:]]       [\x{8140}---΁--O-X`-y--@---ք@-`p---r-]
    \p{JIS}        \pJ        [[:jis:]]         [\p{X0201}\p{X0208}]
-
    \p{NEC}        \pN        [[:nec:]]         [\x{8740}-\x{875D}\x{875f}-\x{8775}\x{877E}-\x{879c}\x{ed40}-\x{eeec}\x{eeef}-\x{eefc}]
    \p{IBM}        \pI        [[:ibm:]]         [\x{fa40}-\x{fc4b}]
    \p{Vendor}     \pV        [[:vendor:]]      [\p{NEC}\p{IBM}]
